<?php
/**
* This module was developed for PrestaShop Addons: http://addons.prestashop.com.
*
* @created   2015-03-20 20:08:00
* @updated   2016-02-11 01:06:00
*
* @author    Daniel Omine <omine@japancase.com>
* @copyright JapanCase - Digital Business
* @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

class Epsilon extends PaymentModule
{
    private $postErrors = array();
	private $html = '';

    private $jc_service_provider = 'Epsilon';

    /*
    決済区分
    */
    private $service_provider_st_code = '11000-0000-00000';

    /*
    支払方法
    10: 一括 Pay entire in one time.
    61: 文割 Deferred payment. In this case, must set an value for 文割数 (pay_time).
    80: リボ Revolving payment / Recurring payment. Automatic Credit Card Billing.
    */
    private $service_provider_card_st_code = '10';
    private $service_provider_card_st_code_deferred = '61';
    private $service_provider_card_st_code_revolving = '80';

    /*
    文割数 Deferred payment time.
    */
    private $service_provider_pay_time = array(1, 3, 5, 6, 10, 12, 15, 18, 20, 24);

    /*
    1: enable credit card code check
    Please, recommended to not change the value.
    */
    private $service_provider_security_check = '1';

    public $store_id = null;
    public $contract_code = null;
    public $store_id_verified = null;
    public $running_card_brand = null;
    public $extra_mail_vars = null;

    /*
    Required by Epsilon.
    Please, do not change.
    */
    const ITEM_CODE = 'PEJPJC';

    /*
    Required by Epsilon.
    Default parameters values
    */

    /*
    課金区分 (月次課金) @see http://www.epsilon.jp/service/credit.html
    If enabled, deferred payment and revolving payment are not acceptable by Epsilon.
    The amount limit for 月次課金 is 30000jpy.
    To enable, set values from 2 up to 10.
    */
    const DEFAULTS_MISSION_CODE = 1;

    const DEFAULTS_CURRENCY_ID = 'JPY';
    const DEFAULTS_PROCESS_CODE = 1;
    const DEFAULTS_XML = 1;
    const DEFAULTS_CHARACTER_CODE = 'UTF8';
    const DEFAULTS_USER_AGENT = 'Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.';

    /*
    Epsilon credit card codes
    */
    const EPSILON_CARD_ID_VISA = 1;
    const EPSILON_CARD_ID_MASTERCARD = 2;
    const EPSILON_CARD_ID_DINERS = 3;
    const EPSILON_CARD_ID_JCB = 4;
    const EPSILON_CARD_ID_AMEX = 5;
    const EPSILON_CARD_ID_DISCOVER = 6; // not available

    /*
    Credit cards id base to be added with Epsilon credit card codes
    ie: Epsilon VISA id = 1
        100 + 1 = 101 -> JapanCase API VISA code = 101
    */
    const EPSILON_CARD_BRAND_ID_BASE = 100;

    /*
    Credit card digits length
    */
    const CREDIT_CARD_MAX_DIGITS = 19;
    const CREDIT_CARD_MIN_DIGITS = 10;

    /*
    Epsilon URL action main parameters
    */
    const EPSILON_API_URL_CARD_PAYMENT = 'direct_card_payment';
    const EPSILON_API_URL_CARD_PAYMENT_MULTI = 'direct_card_payment_multi';
    const EPSILON_API_URL_SALES_PAYMENT = 'sales_payment';
    const EPSILON_API_URL_CHANGE_AMOUNT = 'change_amount_payment';
    const EPSILON_API_URL_CANCEL = 'cancel_payment';

    /*
    Epsilon API available languages
    */
    const API_LANGUAGE_ENGLISH = 'en';
    const API_LANGUAGE_JAPANESE = 'ja';

    /*
    When Epsilon returns billing status as "zero", the transaction has billed.
    */
    const TRANSACTION_STATUS_BILLED = '0';

    /*
    true: enable the use of running_card_brand original value received from get_client_info.
    false: disable it. In this case, EPSILON_RUNNING_CARD_BRAND_POSITION_* will be used.
    Recomended to keep disabled because not returning correctly.
    */
    const EPSILON_ENABLE_ORIGINAL_RUNNING_CARD_BRAND = false;

    /*
    The cards positions into running_st_methods received from get_client_info.
    */
    const EPSILON_RUNNING_CARD_BRAND_POSITION_VISA_MASTERCARD_DINERS = 1;
    const EPSILON_RUNNING_CARD_BRAND_POSITION_JCB_AMEX = 2;

    /*
    The cards information on running_st_methods received from get_client_info.
    1: available for PC only
    2: available for PC and keitai (japanese cellphones)
    */
    const EPSILON_RUNNING_CARD_BRAND_PC = 1;
    const EPSILON_RUNNING_CARD_BRAND_PC_KEITAI = 2;

    /*
    Indicates the option is valid and enabled
    */
    const EPSILON_RUNNING_OPTIONS_FLAG_ENABLED = 1;

    /*
    The options positions into running_option_code received from get_client_info.
    */
    const EPSILON_RUNNING_OPTIONS_POSITION_DEFERRED_REVOLVING = 1;
    const EPSILON_RUNNING_OPTIONS_POSITION_2_TIMES_PAY = 2;
    const EPSILON_RUNNING_OPTIONS_POSITION_3DSECURE = 3;

    /*
    Transaction state codes

    The codes 101, 111, 121, 131 and 211 represents the response from provider.
    Those codes do not indicates if has errors or success.
    In order to check the result, please, read the column "pay_provider_result".
    To check for errors, please, check the column "pay_provider_error_code" and "pay_provider_error_detail".
    (Provider = Epsilon)
    (ACS = Credit Card 3-D Secure authentication company)

    General transactions state codes
    */
    const TRANSACTION_REQUEST  = 10; // Transaction request sent to provider
    const TRANSACTION_RESPONSE = 101; // Transaction response from provider
    const TRANSACTION_AMOUNT_CHANGE_REQUEST = 11; // Transaction amount change request sent to provider
    const TRANSACTION_AMOUNT_CHANGE_RESPONSE = 111; // Transaction amount change response from provider
    const TRANSACTION_CANCEL_REQUEST = 12; // Transaction cancel request sent to provider
    const TRANSACTION_CANCEL_RESPONSE = 121; // Transaction cancel response from provider
    const TRANSACTION_REAL_SALE_REQUEST = 13; // Transaction real sale request sent to provider
    const TRANSACTION_REAL_SALE_RESPONSE = 131; // Transaction real sale response from provider

    /*
    Transaction state codes for 3-D Secure Authentication

    [deprecated] TRANSACTION_3DS_PAREQ_TO_ACS 
    Transaction 3-D Secure PaReq request sent to ACS (credit card company, not the provider)

    [deprecated] TRANSACTION_3DS_PARES_FROM_ACS
    Transaction 3-D Secure PaRes response from ACS (credit card company, not the provider)
    */
    //const TRANSACTION_3DS_PAREQ_TO_ACS = 20;
    //const TRANSACTION_3DS_PARES_FROM_ACS = 201;

    /**
    Transaction 3-D Secure PaRes, received from ACS, being sent to provider
    */
    const TRANSACTION_3DS_PARES_REQUEST = 21;

    /**
    Transaction 3-D Secure PaRes response from provider
    */
    const TRANSACTION_3DS_PARES_RESPONSE = 211;

    /*
    PrestaShop order statuses
    */
    const PS_OS_ACCEPTED = 2;
    const PS_OS_CANCELED = 6;


    /**
    3DS認証結果
    カード会社からの認証結果です。
    イプシロンに送信する「tds_pares」に設定してください。
    */
    const P_PARES = 'PaRes';

    /**
    トラッキングデータ
    セッション管理用のパラメータ
    設定した場合は同じ値が設定されます。

    自由項目
    MD（Merchant Data）は加盟店様のアプリケーションが任意の値を設定できます。
    取引単位にユニークな値を設定してトラッキングにご利用ください。
    ここで設定した値がカード会社からの戻り値にも設定されます。
    データ形式は0x20 から 0x7EまでのASCII文字で、最大データ長は1024バイトです。
    日本語、バイナリなどを含める場合はBase64でエンコードしてください。
    */
    const P_MD = 'MD';

    /**
    戻り値
    戻り値　pareq(Epsilon)　を設定してください。
    */
    const P_PAREQ = 'PaReq';

    /**
    Parameter name of URL will receive the response from ACS.
    戻りURL
    カード会社の認証後に遷移するURLです。
    加盟店様の戻ってくる先を設定してください。
    */
    const P_TERMURL = 'TermUrl';

    /**
    For Epsilon parameter "tds_check_code".
    Use it only when requesting, to Epsilon, the final validation of PaRes parameter, received from 3-D Secure ACS.
    */
    const TDS_CHECK_CODE_VALIDATE_PARES = '2';

    public function __construct()
    {

        $this->name = 'epsilon';
        $this->tab = 'payments_gateways';
        $this->version = '1.1.2';
        $this->author = 'Daniel Omine';
        $this->controllers = array('payment', 'validation');
        $this->ps_versions_compliancy = array('min' => '1.5', 'max' => _PS_VERSION_);
        //$this->is_eu_compatible = 1;
        $this->limited_countries = array('jp');

        $this->module_key = 'f69f2760bf619f95c41e27dc5d39f967';
        $this->currencies = true;
        $this->currencies_mode = 'checkbox';

        $config = Configuration::getMultiple(
            array(
                'EPSILON_JC_STORE_ID',
                'EPSILON_CONTRACT_CODE',
                'EPSILON_TEL',
                'EPSILON_JC_STORE_VERIFIED',
                'EPSILON_RUNNING_CARD_BRAND'
                )
        );

        if (!empty($config['EPSILON_JC_STORE_ID'])) {
            $this->store_id = $config['EPSILON_JC_STORE_ID'];
        }
        if (!empty($config['EPSILON_CONTRACT_CODE'])) {
            $this->contract_code = $config['EPSILON_CONTRACT_CODE'];
        }
        if (!empty($config['EPSILON_TEL'])) {
            $this->contract_code = $config['EPSILON_TEL'];
        }
        if (!empty($config['EPSILON_JC_STORE_VERIFIED'])) {
            $this->store_id_verified = $config['EPSILON_JC_STORE_VERIFIED'];
        }
        if (!empty($config['EPSILON_RUNNING_CARD_BRAND'])) {
            $this->running_card_brand = $config['EPSILON_RUNNING_CARD_BRAND'];
        }

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = $this->l('Epsilon');
        $this->description = $this->l('Credit card payments via Epsilon.');
        $this->confirmUninstall = $this->l('Are you sure about removing these details?');

        if (
            empty($this->store_id)
            || empty($this->contract_code)
            || empty($this->store_id_verified)
            || empty($this->running_card_brand)
        ) {
            $this->warning = $this->l('The store settings must be verified before using this module.');
        }
        if (!count(Currency::checkPaymentCurrencies($this->id))) {
            $this->warning = $this->l('No currency has been set for this module.');
        }

        /*
        $this->extra_mail_vars = array(
            '{epsilon_store_id}' => Configuration::get('EPSILON_JC_STORE_ID'),
            '{epsilon_contract_code}' => nl2br(Configuration::get('EPSILON_CONTRACT_CODE')),
            '{epsilon_jc_store_verified}' => nl2br(Configuration::get('EPSILON_JC_STORE_VERIFIED'))
            );
        */

        $this->ajax_url_base = $this->context->link->getAdminLink('AdminModules', true).'&ajax&configure='.$this->name
        .'&tab_module='.$this->tab.'&module_name='.$this->name
        //.'&token='.Tools::getAdminTokenLite('AdminModules');
        ;
        //echo $this->ajax_url_base; exit;
        /**
        index.php?controller=AdminModules&token=cff046550dc569f7f5fc80c8ea314bc9&ajax&configure=epsilon
        &tab_module=payments_gateways&module_name=epsilon
        */
    }

    public function transactionRequest($customer, $cart)
    {

        include_once(dirname(__FILE__).'/lib/JapanCase.php');
        $jc = new JapanCase();

        $rs = array();
        $rs['error'] = false;

        /*
        [deprecated]
        */
        /*
        $v = Tools::getValue('card');
        $v = $jc->numbersOnly($v);
        if (empty($v))
        {
            $rs['error'] = 010;
            $rs['error_detail'] = $this->l('Please, choose the card');
            return $rs;
        }
        */

        $v = Tools::getValue('card_number');
        $v = $jc->numbersOnly($v);
        $v_l = Tools::strlen($v);
        if (
            empty($v)
            || $v_l > self::CREDIT_CARD_MAX_DIGITS
            || $v_l < self::CREDIT_CARD_MIN_DIGITS
        ) {
            $rs['error'] = 010;
            $rs['error_detail'] = $this->l('The card number seems invalid.');
            return $rs;
        }
        $card_number = $v;

        /*
        Auto detect the card brand
        */
        $card = $this->cardType($v);
        //print_r($card); exit;
        if (!$card->available) {
            $rs['error'] = 011;

            if (!empty($card->id)) {
                $arr = array(
                    $this->l('The card information was denied.'),
                    sprintf($this->l('%s card is not accepted.'), $card->brand)
                );
                //$rs['error_detail'] = json_encode($arr);
                $rs['error_detail'] = Tools::jsonEncode($arr);
            } else {
                $rs['error_detail'] = $this->l('The card number was not recognized.');
            }

            //print_r($rs['error_detail']); exit;
            return $rs;
        }

        $jc_pay_method = $card->id;
        //print_r($card); exit;


        $v = Tools::getValue('card_expire_year');
        $v = $jc->numbersOnly($v);
        if (
            (empty($v)
            || Tools::strlen($v) != 4)
            && $v < date('Y')
        ) {
            $rs['error'] = 012;
            $rs['error_detail'] = $this->l('The card expire date (year) seems invalid.');
            return $rs;
        }
        $card_expire_year = $v;

        $v = Tools::getValue('card_expire_month');
        $v = $jc->numbersOnly($v);
        if (
            (empty($v)
            || $v < 1
            || $v > 12)
            || ($card_expire_year == date('Y') && $v < date('n'))
        ) {
            $rs['error'] = 013;
            $rs['error_detail'] = $this->l('The card expire date (month) seems invalid.');
            return $rs;
        }
        $card_expire_month = $v;

        $v = Tools::getValue('card_code');
        $v = $jc->numbersOnly($v);
        if (
            (empty($v)
            || Tools::strlen($v) < 3
            || Tools::strlen($v) > 4)
        ) {
            $rs['error'] = 014;
            $rs['error_detail'] = $this->l('The card security code seems invalid.');
            return $rs;
        }
        $card_code = $v;

        $card_st_code = $this->service_provider_card_st_code;

        if ($this->optionsPayTime()) {
            $v = Tools::getValue('card_pay_time');
            $v = $jc->numbersOnly($v);
            if (
                empty($v)
                || !in_array($v, $this->optionsPayTime())
            ) {
                $rs['error'] = 015;
                $rs['error_detail'] = $this->l('The deferred pay time seems invalid.');
                return $rs;
            }
            $card_pay_time = $v;

            if ($v > 1) {
                $card_st_code = $this->service_provider_card_st_code_deferred;
            }
        }

        if ($this->optionsRevolving()) {
            $card_st_code = $this->service_provider_card_st_code_revolving;
        }

        /**
        Parameters to be sent
        */
        $rs['data'] = array(
            'p'                    => 'Transaction/Send',
            'jc_service_provider'  => $this->jc_service_provider,
            'jc_action_url'        => self::EPSILON_API_URL_CARD_PAYMENT,
            'jc_store_id'          => Configuration::get('EPSILON_JC_STORE_ID'),
            'jc_debug_urls'        => Configuration::get('EPSILON_JC_DEBUG_URLS'),
            'jc_pay_method'        => $jc_pay_method,
            'jc_ps_id_cart'        => $cart->id, // PrestaShop Cart ID

            /*
            The order number must be changed in case of Epsilon denies authentication.
            When a transaction authenticatin fails, Epsilon do not permit send again the same order number.
            */
            'order_number'         => $this->newID(),
            'contract_code'        => Configuration::get('EPSILON_CONTRACT_CODE'),

            /*
            The value of "tds_check_code" must be 1 to request 3-D Secure authentication with Epsilon.
            In case of empty value, the 3-D Secure will be ignored.
            */
            'tds_check_code'       => Configuration::get('EPSILON_3DS_CHECK_CODE'),

            /*
            The API language for messages returned.
            */
            'lang_id'              => Configuration::get('EPSILON_API_LANGUAGE'),
            'user_id'              => $customer->id,
            'user_name'            => $customer->lastname.$customer->firstname,
            'user_mail_add'        => $customer->email,
            'item_price'           => round($cart->getOrderTotal(true, Cart::BOTH)),
            'item_code'            => self::ITEM_CODE,
            'item_name'            => Configuration::get('PS_SHOP_NAME'),
            'currency_id'          => self::DEFAULTS_CURRENCY_ID,
            'card_number'          => $card_number,
            'expire_y'             => $card_expire_year,
            'expire_m'             => $card_expire_month,
            'card_st_code'         => $card_st_code,
            'security_check'       => $this->service_provider_security_check,
            'security_code'        => $card_code,
            'memo1'                => '',
            'memo2'                => '',
            'st_code'              => $this->service_provider_st_code,
            'mission_code'         => self::DEFAULTS_MISSION_CODE,
            'process_code'         => self::DEFAULTS_PROCESS_CODE,
            'xml'                  => self::DEFAULTS_XML,
            'character_code'       => self::DEFAULTS_CHARACTER_CODE,
            'user_agent'           => self::DEFAULTS_USER_AGENT
        );

        if ($card_st_code == $this->service_provider_card_st_code_deferred) {
            $rs['data']['pay_time'] = $card_pay_time;
        }

        //print_r($rs['data']); exit;
        $rs['order'] = $this->orderSave($rs['data']);

        if (!$rs['order']['error']) {
            /*
            Saving the order data to the log table.
            */
            //echo 'saved';

            $rs['gateway'] = $jc->transactionSend($rs['data']);

            /** 
            Saving the provider response to the log table, regardless if returned error or not.
            */
            $this->orderSaveProviderResponse($rs['data'], $rs['gateway']->provider);

            /*
            Note: The acs_url not implemented (3D-S).
            */
            if (
                (!isset($rs['gateway']->provider['error_code']) || empty($rs['gateway']->provider['error_code']))
                && isset($rs['gateway']->provider['transaction_id'])
                && !empty($rs['gateway']->provider['transaction_id'])
                && ((int)$rs['gateway']->provider['result'] == 1 || (int)$rs['gateway']->provider['result'] == 5)
                //&& (!isset($rs['gateway']->provider['acs_url']) || empty($rs['gateway']->provider['acs_url']))
            ) {

            } else {

                //$rs['ps_order_status'] = 'EPSILON_OS_ERROR'; // [deprecated]
                $rs['error'] = 101;
                $rs['error_detail'] = $this->l('Error returned from payment gateway provider.'); // From Epsilon

            }

        } else {
            /*
            Order not saved to log table.
            */
            //echo 'error';
            //$rs['ps_order_status'] = 'PS_OS_ERROR'; // [deprecated]
            $rs['error'] = 001;
            $rs['error_detail'] = $this->l('Error occurred on log table.');
        }

        return $rs;

    }

    public function hookDisplayPDFInvoice($params)
    {

        //print_r($params['object']); exit;
        $orderInvoice = $params['object'];

        $sql = 'SELECT 
        T1.id_cart AS `id_cart`,
        T2.client_pay_method AS `client_pay_method`
        FROM `'._DB_PREFIX_.'orders` AS T1 
        LEFT JOIN `'._DB_PREFIX_.'epsilon_client_order` AS T2 
        ON T2.ps_id_cart = T1.id_cart AND T2.client_pay_method IS NOT NULL
        WHERE T1.id_order = '.(int)($orderInvoice->id_order);

        $rs = Db::getInstance()->getRow($sql);

        if (!empty($rs) && isset($rs['client_pay_method']) && !empty($rs['client_pay_method'])) {
            $cards = $this->runningCards();
            //echo $cards[$rs['client_pay_method']]['brand']; exit;

            $this->smarty->assign('card', $cards[$rs['client_pay_method']]);
            return $this->display(__FILE__, 'pdf-invoice.tpl');
        }

    }

    private function hooksList()
    {
        return array('payment','paymentReturn','displayPDFInvoice','displayAdminOrder');
    }

    public function install()
    {
        if (!parent::install()) {
            return false;
        }

        $hooks = $this->hooksList();

        foreach ($hooks as $v) {
            if (!$this->registerHook($v)) {
                return false;
            }
        }

        include_once(_PS_MODULE_DIR_.'/'.$this->name.'/epsilon_install.php');
        $install = new EpsilonInstall();
        $install->createTables();
        $install->setConfiguration();
        $install->createOrderState();

        return true;
    }

    public function uninstall()
    {

        include_once(_PS_MODULE_DIR_.'/'.$this->name.'/epsilon_install.php');
        $install = new EpsilonInstall();

        if (
            !$install->deleteConfiguration()
            || !parent::uninstall()
        ) {
            return false;
        }

        $hooks = $this->hooksList();

        foreach ($hooks as $v) {
            $this->unregisterHook($v);
        }

        return true;
    }

    private function postValidation()
    {
        if (Tools::isSubmit('btnSubmit')) {

            include_once(dirname(__FILE__).'/lib/JapanCase.php');
            $jc = new JapanCase;

            /*
            PrestaShop store url
            */
            $v = Tools::getValue('EPSILON_JC_STORE_URL');
            $v = str_replace(array('http://','https://'), '', $v);
            if (empty($v)) {
                $this->postErrors[] = $this->l('Store URL is required.');
            }
            $store_url = $v;

            /*
            PrestaShop store email
            */
            $v = Tools::getValue('EPSILON_JC_STORE_EMAIL');
            //echo $v; exit;
            if (empty($v)) {
                $this->postErrors[] = $this->l('Store email is required.');
            }
            if (!$jc->validEmailFormat($v)) {
                $this->postErrors[] = $this->l('Store email format is invalid.');
            }
            $store_email = $v;

            /*
            PrestaShop store name
            */
            $v = Tools::getValue('EPSILON_JC_STORE_NAME');
            if (empty($v)) {
                $this->postErrors[] = $this->l('Store name is required.');
            }
            $store_name = $v;

            /*
            Epsilon requires the 4 last digits
            */
            $v = $jc->numbersOnly(Tools::getValue('EPSILON_CONTRACT_CODE'));
            if (empty($v)) {
                $this->postErrors[] = $this->l('Contract code is required.');
            }
            $contract_code = $v;

            /*
            Epsilon requires the 4 last digits
            */
            $v = Tools::substr($jc->numbersOnly(Tools::getValue('EPSILON_TEL')), -4);
            if (
                empty($v)
                || Tools::strlen($v) != 4
            ) {
                $this->postErrors[] = $this->l('Tel number, registered with Epsilon, is required.');
            }
            $tel = $v;

            /*
            Flag to indicates new validation or revalidation.
            */
            $revalidate_settings  = Tools::getValue('REVALIDATE_SETTINGS_1');



            /*
            Checking if the data is already verified to avoid resend same request.
            However, if $revalidate_settings is true, the data must be revalidated with Epsilon and JapanCase api.
            */
            $config = Configuration::getMultiple(
                array(
                    'EPSILON_JC_STORE_ID',
                    'EPSILON_JC_STORE_URL',
                    'EPSILON_JC_STORE_EMAIL',
                    'EPSILON_JC_STORE_NAME',
                    'EPSILON_JC_STORE_VERIFIED',
                    'EPSILON_RUNNING_CARD_BRAND'
                )
            );
    
            $config['EPSILON_JC_STORE_URL'] = str_replace(
                array('http://','https://'),
                '',
                $config['EPSILON_JC_STORE_URL']
            );

            if (
                empty($this->postErrors)
                && $revalidate_settings
                || (!$config['EPSILON_JC_STORE_VERIFIED'] || empty($config['EPSILON_JC_STORE_VERIFIED']))
                || (!$config['EPSILON_RUNNING_CARD_BRAND'] || empty($config['EPSILON_RUNNING_CARD_BRAND']))
            ) {

                /*
                Validating the store ID from JapanCase API            
                */
                include_once(dirname(__FILE__).'/lib/JapanCase.php');
                $jc = new JapanCase();
                $store = $jc -> ValidateCode($contract_code, $tel, $store_name, $store_url, $store_email);

                /** Debug purposes */
                //print_r($store); exit;
                //print_r($store->provider['data']['running_card_brand']); exit;

                if (is_array($store -> error)) {

                    foreach ($store -> error as $k => $v) {
                        $this->postErrors[] = $this->l('"'.$k.'": '.$v);
                    }

                } elseif (
                    $store -> error
                    && !empty($store -> error)
                ) {

                    if (
                        isset($store -> error_detail)
                        && !empty($store -> error_detail)
                    ) {

                        if (is_array($store -> error_detail)) {
                            foreach ($store -> error_detail as $k => $v) {
                                $this->postErrors[] = $this->l('"'.$k.'": '.$v);
                            }
                        } else {
                            $this->postErrors[] = $store -> error_detail;
                        }

                    }

                } elseif (!empty($store->provider['error'])) {

                    if (is_array($store->provider['error'])) {
                        foreach ($store->provider['error'] as $k => $v) {
                            $this->postErrors[] = $this->l('"'.$k.'": '.$v);
                        }
                    } else {
                        $this->postErrors[] = $this->l('provider: '.$store->provider['error']);
                    }

                } elseif (!empty($store->provider['curl_error_code'])) {

                    if (is_array($store->provider['curl_error_code'])) {
                        foreach ($store->provider['curl_error_code'] as $k => $v) {
                            $this->postErrors[] = $this->l('"'.$k.'": '.$v);
                        }
                    } else {
                        $this->postErrors[] = $this->l('Provider curl:').' '.$store->provider['curl_error_code'];
                    }

                } elseif (
                    isset($store->provider['data']['err_code'])
                    && !empty($store->provider['data']['err_code'])
                ) {

                    $this->postErrors[] = $this->l('Epsilon error code:').' '.$store->provider['data']['err_code'];
                    $this->postErrors[] = $this->l('Epsilon error detail:').' '.$store->provider['data']['err_detail'];

                } elseif (
                    isset($store->provider['data']['cancel_date'])
                    && !empty($store->provider['data']['cancel_date'])
                ) {

                    /*
                    加盟店様状態　2の場合設定　1,3の場合はNULL
                    イプシロンに加盟店様との契約を解除した
                    （決済サービスを利用できなくなった）年月日が
                    設定
                    形式はYYYYMMDD
                    */
                    $this->postErrors[] = $this->l('Epsilon cancel date:').' '.$store->provider['data']['cancel_date'];

                } elseif (
                    isset($store->provider['data']['client_state'])
                    && $store->provider['data']['client_state'] != 1
                ) {

                    /*
                    イプシロンでの契約の状態
                    1：契約中　イプシロンの決済サービスが利用できる状態です
                    2：解約済　イプシロンとの契約が解除されています　
                    決済サービスが利用できない状態です
                    3：仮契約　イプシロンに申込み中で審査中です
                    決済サービスが利用できない状態です
                    */
                    $this->postErrors[] = $this->l('Epsilon client state: '.$store->provider['data']['client_state']);

                } elseif (
                    isset($store->provider['data']['license_check_flag'])
                    && $store->provider['data']['license_check_flag'] != 1
                ) {

                    /*
                    イプシロンにて特定の商材に対する必要な免許を確認したか否かのフラグ
                    0：未チェック
                    1：チェック済み
                    */
                    $this->postErrors[] = $this->l('Epsilon license not checked').
                    ' ('.$store->provider['data']['license_check_flag'].')';

                } elseif (
                    self::EPSILON_ENABLE_ORIGINAL_RUNNING_CARD_BRAND
                    && (
                        !isset($store->provider['data']['running_card_brand'])
                        || empty($store->provider['data']['running_card_brand'])
                    )
                ) {

                    /*
                    加盟店様状態　1の場合設定　2,3の場合はNULL
                    各桁が利用可能なクレジットカードブランドに対応
                    左から　1：VISA、2：MasterCard、3：Diners、4：JCB、5：AMEX　に対応
                    利用可能な場合は1、利用不可の場合は0を設定
                    例)　11000 の場合　VISA、MasterCardは利用可能　Diners、JCB、AMEX　は利用不可
                    */
                    $this->postErrors[] = $this->l('Epsilon credit card brand missing');

                }
                
                if (empty($this->postErrors)) {

                    /**
                    The store id was verified successfuly.
                    However, the contract code still must be validated with Epsilon.
                    */
                    Configuration::updateValue('EPSILON_JC_STORE_ID', $store->data['id']);
                    Configuration::updateValue('EPSILON_JC_STORE_VERIFIED', time());

                    /*
                    稼働中決済方法、審査中決済方法 running_st_methods
                    20000-0000-00002

                        0：未稼働
                        1：稼動（PC のみ利用可）
                        2：稼動（PC・携帯で利用可）

                    1.  クレジットカード決済（VISA、MasterCard、Diners）
                    2.  クレジットカード決済（JCB、AMEX）
                    3.  コンビニ決済
                    4.  ネットバンク決済（ジャパンネット銀行）
                    5.  ネットバンク決済（楽天銀行）
                    6.  代引き（西濃運輸）
                    7.  ネットバンク決済（ペイジー）
                    8.  電子マネー（WebMoney）
                    9.  ウォレット（Yahoo!ウォレット）
                    10. 未使用
                    11. ウォレット（PayPal）
                    12. 電子マネー（BitCash）
                    */
                    Configuration::updateValue(
                        'EPSILON_RUNNING_ST_METHODS',
                        $store->provider['data']['running_st_methods']
                    ); //running_st_code

                    /*
                    稼働中オプション・配送、審査中オプション・配送 running_option_code

                        0：未稼働
                        1：稼動

                    1.  分割払い/リボルビング払い
                    2.  ２回締め
                    3.  ３Ｄセキュア認証支援サービス
                    4.  ３Ｄセキュア認証支援サービス(JCB必須)
                    5.  ３Ｄセキュア認証支援サービス(VM必須)
                    6.  決済承認通知メール
                    7.  配送：ゆうパック
                    8.  SNS-APL対応
                    9.  配送：西濃運輸
                    10. ? チャージバック保証サービス利用料
                    11. ? GMO後払い(請求書タイプ：はがき)
                    12. 早期入金サービス利用料 Early credit
                    */
                    Configuration::updateValue(
                        'EPSILON_RUNNING_OPTION_CODE',
                        $store->provider['data']['running_option_code']
                    );

                    if (!self::EPSILON_ENABLE_ORIGINAL_RUNNING_CARD_BRAND) {
                        /**
                        Reading information for VISA, MASTERCARD and DINERS

                        The position on string
                        */
                        $p = self::EPSILON_RUNNING_CARD_BRAND_POSITION_VISA_MASTERCARD_DINERS;

                        /**
                        The position must be decreased because array starts on zero
                        */
                        $v = $store->provider['data']['running_st_methods'][($p-1)];

                        /**
                        Saving information for VISA, MASTERCARD and DINERS
                        */
                        $running_card_brand = $v.$v.$v;

                        /*
                        Reading information for JCB and AMEX

                        The position on string
                        */
                        $p = self::EPSILON_RUNNING_CARD_BRAND_POSITION_JCB_AMEX;

                        /**
                        The position must be decreased because array starts on zero
                        */
                        $v = $store->provider['data']['running_st_methods'][($p-1)];

                        /**
                        Saving information for JCB and AMEX
                        */
                        $running_card_brand .= $v.$v;

                    } else {
                        /*
                        [important]
                        Keep disable because the value returned by Epsilon seems not correct.
                        Returning 11000 instead of 11100.
                        If running_st_methods first char is 1 or 2, 
                        means the running_card_brand should be 111 on first 3 chars, 
                        but this is not happens as expected.
                        This is the reason to avoid use of running_card_brand value.

                        $store->provider['data']['running_card_brand'] is deprecated.
                        The information about running cards is on "running_st_methods".
                        If value is 1 or 2, means: available. If 0: unavailable.
                            0：未稼働
                            1：稼動（PC のみ利用可）
                            2：稼動（PC・携帯で利用可）
                        */
                        $running_card_brand = $store->provider['data']['running_card_brand'];

                        /*
                        [deprecated]
                        if ($cards = $this->runningCards($store->provider['data']['running_card_brand']))
                        {
                            foreach($cards as $k => $v)
                            {
                                Configuration::updateValue('EPSILON_CARD_ENABLED_'.($k), 1);
                            }
                        }
                        */
                    }

                    Configuration::updateValue(
                        'EPSILON_RUNNING_CARD_BRAND',
                        $running_card_brand
                    );
                    Configuration::updateValue(
                        'EPSILON_RUNNING_CARD_BRAND_ORIGINAL',
                        $store->provider['data']['running_card_brand']
                    );

                } else {
                    Configuration::deleteByName('EPSILON_JC_STORE_VERIFIED');
                }

            }

            if (empty($this->postErrors)) {
                include_once(_PS_MODULE_DIR_.'/'.$this->name.'/epsilon_install.php');
                $install = new EpsilonInstall();
                $install->updateConfiguration();
                //$this->postErrors[] = $this->l('The settings was saved');
                Configuration::deleteByName('EPSILON_VERIFICATION_ERRORS');
            } else {
                Configuration::updateValue(
                    'EPSILON_VERIFICATION_ERRORS',
                    //json_encode($this->postErrors)
                    Tools::jsonEncode($this->postErrors)
                );
            }

        }
    }

    /*
    Verifies if credit cards are available for current contract
    */
    public function epsilonCards($id = null)
    {
        /*
        加盟店様状態　1の場合設定　2,3の場合はNULL
        各桁が利用可能なクレジットカードブランドに対応
        左から　1：VISA、2：MasterCard、3：Diners、4：JCB、5：AMEX　に対応
        利用可能な場合は1、利用不可の場合は0を設定
        例)　11000 の場合　VISA、MasterCardは利用可能　Diners、JCB、AMEX　は利用不可
        
        The array values must be in this exact order.
        */
        $rs = array(
            array('brand'=>'VISA','ico'=>'visa.png'),
            array('brand'=>'MasterCard','ico'=>'mastercard.png'),
            array('brand'=>'Diners','ico'=>'diners.png'),
            array('brand'=>'JCB','ico'=>'jcb.png'),
            array('brand'=>'AMEX','ico'=>'amex.png')
        );

        if (empty($id)) {
            return $rs;
        }

        if (isset($rs[$id])) {
            return $rs[$id];
        }

        return null;
    }

    /*
    Verifies if option is available
    @param $option values:
        self::EPSILON_RUNNING_OPTIONS_POSITION_DEFERRED_REVOLVING
        self::EPSILON_RUNNING_OPTIONS_POSITION_2_TIMES_PAY
        self::EPSILON_RUNNING_OPTIONS_POSITION_3DSECURE
    */
    public function runningOptions($option = null)
    {

        if (empty($option)) {
            return false;
        }

        $running_options = Configuration::get('EPSILON_RUNNING_OPTION_CODE');

        $option--;

        if (
            isset($running_options[$option])
            && $running_options[$option] == self::EPSILON_RUNNING_OPTIONS_FLAG_ENABLED
        ) {
            return true;
        }

        return false;
    }

    /*
    Verifies if credit cards are available for current contract
    */
    public function runningCards($running_card_brand = false)
    {

        $rs = false;

        if (!$running_card_brand) {
            /*
            Check on database the available card brands
            */
            $running_card_brand =  Configuration::get('EPSILON_RUNNING_CARD_BRAND');
        }

        /*
        加盟店様状態　1の場合設定　2,3の場合はNULL
        各桁が利用可能なクレジットカードブランドに対応
        左から　1：VISA、2：MasterCard、3：Diners、4：JCB、5：AMEX　に対応
        利用可能な場合は1、利用不可の場合は0を設定
        例)　11000 の場合　VISA、MasterCardは利用可能　Diners、JCB、AMEX　は利用不可
        */
        $cards = $this->epsilonCards();

        foreach ($cards as $k => $v) {
            if (
                $running_card_brand[$k] == self::EPSILON_RUNNING_CARD_BRAND_PC
                || $running_card_brand[$k] == self::EPSILON_RUNNING_CARD_BRAND_PC_KEITAI
            ) {
                $rs[(self::EPSILON_CARD_BRAND_ID_BASE+$k+1)] = $v;
            }
        }

        return $rs;

    }

    /**
    * Return credit card type if number is valid.
    * Returns null if credit card brand not available for current contract code.@see self::runningCards()
    * @return string
    * @param $number string
    **/
    public function cardType($number = null)
    {
        if (empty($number)) {
            return null;
        }

        $number=preg_replace('/[^\d]/', '', $number);
        if (preg_match('/^3[47][0-9]{13}$/', $number)) {
            /**
            American Express
            */
            $id = self::EPSILON_CARD_ID_AMEX;
        } elseif (preg_match('/^3(?:0[0-5]|[68][0-9])[0-9]{11}$/', $number)) {
            /**
            Diners Club
            */
            $id = self::EPSILON_CARD_ID_DINERS;
        } elseif (preg_match('/^6(?:011|5[0-9][0-9])[0-9]{12}$/', $number)) {
            // Discover
            $id = self::EPSILON_CARD_ID_DISCOVER;
        } elseif (preg_match('/^(?:2131|1800|35\d{3})\d{11}$/', $number)) {
            // JCB
            $id = self::EPSILON_CARD_ID_JCB;
        } elseif (preg_match('/^5[1-5][0-9]{14}$/', $number)) {
            // MasterCard
            $id = self::EPSILON_CARD_ID_MASTERCARD;
        } elseif (preg_match('/^4[0-9]{12}(?:[0-9]{3})?$/', $number)) {
            // Visa
            $id = self::EPSILON_CARD_ID_VISA;
        } else {
            $id = null;
        }

        $cards = $this->runningCards();

        $card_id = (self::EPSILON_CARD_BRAND_ID_BASE + $id);

        $rs = (object) array();
        if (isset($cards[$card_id])) {
            $rs->available = true;
            $rs->id = $card_id;
            $rs->number = $number;
            foreach ($cards[$card_id] as $k => $v) {
                $rs->{$k} = $v;
            }
        } else {
            $rs->available = false;
            $rs->id = (empty($id)?$id:$card_id);
            $rs->number = $number;
            if (!empty($id)) {
                $cards = $this->epsilonCards(($id-1));
                foreach ($cards as $k => $v) {
                    $rs->{$k} = $v;
                }
            }
        }
        //print_r($rs); exit;
        return $rs;
    }


    /*
    Referers to EpsilonInstall::orderStates()
    file epsilon_install.php
    */
    private function cardJCCodeToOrderStatusCode($jc_card_code = false)
    {
        $rs = array(
            (self::EPSILON_CARD_BRAND_ID_BASE + self::EPSILON_CARD_ID_VISA) => 'EPSILON_OS_CC_VISA',
            (self::EPSILON_CARD_BRAND_ID_BASE + self::EPSILON_CARD_ID_MASTERCARD) => 'EPSILON_OS_CC_MASTERCARD',
            (self::EPSILON_CARD_BRAND_ID_BASE + self::EPSILON_CARD_ID_DINERS) => 'EPSILON_OS_CC_DINERS_CLUB',
            (self::EPSILON_CARD_BRAND_ID_BASE + self::EPSILON_CARD_ID_JCB) => 'EPSILON_OS_CC_JCB',
            (self::EPSILON_CARD_BRAND_ID_BASE + self::EPSILON_CARD_ID_AMEX) => 'EPSILON_OS_CC_AMEX'
        );

        if ($jc_card_code) {
            return $rs[$jc_card_code];
        }

        return $rs;
    }


    /*
    Check if revolving payment is enabled.
    [deprecated]
    public function twoTimesPayment()
    {
        if (!Configuration::get('EPSILON_2_TIMES_PAY'))
            return null;

        return true;
    }
    */


    /*
    Check if revolving payment is enabled.
    */
    public function optionsRevolving()
    {
        if (!Configuration::get('EPSILON_REVOLVING_PAYMENT')) {
            return null;
        }

        return true;
    }


    /*
    Returns the array of available time options for deferred payments.
    If not enabled, returns null.
    */
    public function optionsPayTime()
    {
        if (!Configuration::get('EPSILON_DEFERRED_PAYMENT')) {
            return null;
        }

        return $this->service_provider_pay_time;
    }

    private function postProcess()
    {
        if (Tools::isSubmit('btnSubmit')) {
            include_once(_PS_MODULE_DIR_.'/'.$this->name.'/epsilon_install.php');
            $install = new EpsilonInstall();
            $install->updateConfiguration();
        }
        $this->html .= $this->displayConfirmation($this->l('Settings updated'));
    }

    private function displayEpsilon()
    {
        //echo ':'.$this->author.':'; exit;
        $this->context->controller->addCSS(
            Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/views/css/epsilon.css'
        );

        if ($running_card_brand = Configuration::get('EPSILON_RUNNING_CARD_BRAND')) {
            $running_card_brand = (!empty($running_card_brand) ? $this->runningCards($running_card_brand) : '');
        }

        $this->store_id_verified = Configuration::get('EPSILON_JC_STORE_VERIFIED');

        $this->smarty->assign(array(
            'jc_store_verified' => (
                (!empty($this->store_id_verified)) ? $this->l(date('Y-m-d H:i:s', $this->store_id_verified)) : false
            ),
            'running_card_brand' => ((!empty($running_card_brand)) ? $running_card_brand : false),
            'frm_epsilon_3ds_check_code' => 'EPSILON_3DS_CHECK_CODE_1',
            'epsilon_running_options_3dsecure' => (
                $this->runningOptions(self::EPSILON_RUNNING_OPTIONS_POSITION_3DSECURE) ? 'false':'true'
            ),
            'frm_epsilon_deferred_payment' => 'EPSILON_DEFERRED_PAYMENT_1',
            'frm_epsilon_revolving_payment' => 'EPSILON_REVOLVING_PAYMENT_1',
            'epsilon_running_options_position_deferred_revolving' => (
                $this->runningOptions(self::EPSILON_RUNNING_OPTIONS_POSITION_DEFERRED_REVOLVING) ? 'false':'true'
            )
            //'frm_epsilon_2_times_pay' => 'EPSILON_2_TIMES_PAY_1',
            /**
            'epsilon_running_options_position_2_times_pay' => (
            $this->runningOptions(self::EPSILON_RUNNING_OPTIONS_POSITION_2_TIMES_PAY) ? 'false':'true'
            ),
            */
        ));

        return $this->display(__FILE__, 'infos.tpl');
    }

    public function getContent()
    {
        if (Tools::isSubmit('btnSubmit')) {
            $this->postValidation();
            if (!count($this->postErrors)) {
                $this->postProcess();
            } else {
                foreach ($this->postErrors as $err) {
                    $this->html .= $this->displayError($err);
                }
            }
        } else {
            $this->html .= '<br />';
        }

        $this->html .= $this->displayEpsilon();
        $this->html .= $this->renderForm();

        return $this->html;
    }

    public function hookPayment($params)
    {
        if (!$this->active) {
            return;
        }
        if (!$this->checkCurrency($params['cart'])) {
            return;
        }
        if (!$cards = $this->runningCards()) {
            return;
        }
        $this->smarty->assign(array(
            'cards' => $cards,
            'this_path' => $this->_path,
            'this_path_bw' => $this->_path,
            'this_path_ssl' => Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/'
        ));
        return $this->display(__FILE__, 'payment.tpl');
    }

    public function hookPaymentReturn($params)
    {
        if (!$this->active) {
            return;
        }

        $state = $params['objOrder']->getCurrentState();
        /**
        if (in_array(
            $state,
            array(
                Configuration::get('EPSILON_JC_STORE_ID'),
                Configuration::get('PS_OS_OUTOFSTOCK'),
                Configuration::get('PS_OS_OUTOFSTOCK_UNPAID')
		    )
        ))
        */
        if (in_array(
            $state,
            array(
                Configuration::get('EPSILON_OS_ACCEPTED'),
                Configuration::get('EPSILON_OS_PROCESSING')
            )
        )) {
            $this->smarty->assign(array(
                'total_to_pay' => Tools::displayPrice(
                    $params['total_to_pay'],
                    $params['currencyObj'],
                    false
                ),
                'status' => 'ok',
                'id_order' => $params['objOrder']->id
            ));
            if (
                isset($params['objOrder']->reference)
                && !empty($params['objOrder']->reference)
            ) {
                $this->smarty->assign('reference', $params['objOrder']->reference);
            }
        } else {
            $this->smarty->assign('status', 'failed');
        }

        return $this->display(__FILE__, 'payment_return.tpl');
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }

        return false;
    }

    public function renderForm()
    {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Payment gateway settings'),
                    'icon' => 'icon-cogs'
                ),
                'input' => array(
                    array(
                        'type' => 'text',
                        'label' => $this->l('Contract Code'),
                        'name' => 'EPSILON_CONTRACT_CODE',
                        'desc' => $this->l(
                            'Epsilon contract code is required to receive the Store ID and activates the module'
                        ),
                        'required' => true
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Tel number'),
                        'name' => 'EPSILON_TEL',
                        'desc' => $this->l('The tel number registered with Epsilon.'),
                        'required' => true
                    ),
                    array(
                        'type' => 'checkbox',
                        //'label' => $this->l('TEST MODE'),
                        //'is_bool' => true,
                        'name' => 'EPSILON_JC_DEBUG_URLS',
                        'values' => array(
                            'query' => array(array(
                                'id'=>'1',
                                'name'=>$this->l('Test mode'),
                                'val'=>1
                            )),
                            'id' => 'id',
                            'name' => 'name',
                        ),
                        'desc' => $this->l(
                            'Check this box to enable test mode. Uncheck the box to enable the production mode.'
                        ),
                        'required' => true
                    ),
                    array(
                        'type' => 'checkbox',
                        'name' => 'EPSILON_DEFERRED_PAYMENT',
                        'values' => array(
                            'query' => array(array(
                                'id'=>'1',
                                'name'=>$this->l('Deferred Payment'),
                                'val'=>1
                            )),
                            'id' => 'id',
                            'name' => 'name',
                        ),
                        'desc' => $this->l(
                            'Check this box to enable deferred payment if available to your contract with Epsilon.'
                        ),
                        'required' => true
                    ),
                    array(
                        'type' => 'checkbox',
                        'name' => 'EPSILON_REVOLVING_PAYMENT',
                        'values' => array(
                            'query' => array(array(
                                'id'=>'1',
                                'name'=>$this->l('Revolving Payment'),
                                'val'=>1
                            )),
                            'id' => 'id',
                            'name' => 'name',
                        ),
                        'desc' => $this->l(
                            'Check this box to enable revolving payment if available to your contract with Epsilon.'
                        ),
                        'required' => true
                    ),
                    /*
                    array(
                        'type' => 'checkbox',
                        'name' => 'EPSILON_2_TIMES_PAY',
                        'values' => array(
                                    'query' => array(array('id'=>'1','name'=>$this->l('2 times payment'),'val'=>1),
                                                ),
                                    'id' => 'id',
                                    'name' => 'name',
                                    ),
                        'desc' => $this->l(
                            'Check this box to enable 2 times payment if available to your contract with Epsilon.'
                        ),
                        'required' => true
                    ),
                    */
                    array(
                        'type' => 'checkbox',
                        'name' => 'EPSILON_3DS_CHECK_CODE',
                        'values' => array(
                            'query' => array(array(
                                'id'=>'1',
                                'name'=>$this->l('3-D Secure'),
                                'val'=>1
                            )),
                            'id' => 'id',
                            'name' => 'name',
                        ),
                        'desc' => $this->l(
                            'Check this box to enable 3-D Security authentication 
                            if available to your contract with Epsilon.'
                        ),
                        'required' => true
                    ),

                    array(
                        'type' => 'select',
                        'name' => 'EPSILON_API_LANGUAGE',
                        'options' => array(
                            'query' => array(
                                array(
                                    'id'=>self::API_LANGUAGE_ENGLISH,
                                    'name'=>$this->l('English')
                                ),
                                array(
                                    'id'=>self::API_LANGUAGE_JAPANESE,
                                    'name'=>$this->l('Japanese')
                                ),
                            ),
                            'id' => 'id',
                            'name' => 'name'
                        ),
                        'label' => $this->l('Epsilon API language'),
                        'required' => true
                    ),

                    array(
                        'type' => 'text',
                        'label' => $this->l('Store ID'),
                        'name' => 'EPSILON_JC_STORE_ID',
                        'required' => true,
                        'readonly' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Store Name'),
                        'name' => 'EPSILON_JC_STORE_NAME',
                        'readonly' => true,
                        'required' => true
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Store URL'),
                        'name' => 'EPSILON_JC_STORE_URL',
                        'readonly' => true,
                        'required' => true
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Store Email'),
                        'name' => 'EPSILON_JC_STORE_EMAIL',
                        'readonly' => true,
                        'required' => true
                    ),
                ),
                'submit' => array('title' => $this->l('Save'))
            ),
        );

        if ($this->store_id_verified) {
            $fields_form['form']['input'][] = array(
                'type' => 'checkbox',
                'name' => 'REVALIDATE_SETTINGS',
                'values' => array(
                    'query' => array(array(
                        'id'=>'1',
                        'name'=>$this->l('Settings verification'),
                        'val'=>1
                    )),
                    'id' => 'id',
                    'name' => 'name',
                ),
                'desc' => $this->l('Check this box to revalidate the settings with Epsilon api.'),
                'required' => true
            );
        }

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = (
            Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get(
                'PS_BO_ALLOW_EMPLOYEE_FORM_LANG'
            ) : 0
        );
        $this->fields_form = array();
        $helper->id = (int)Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'btnSubmit';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.
        '&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );


        return $helper->generateForm(array($fields_form));
    }

    public function getConfigFieldsValues()
    {
        $rs = array(
            'EPSILON_JC_STORE_NAME' => Tools::getValue(
                'EPSILON_JC_STORE_NAME',
                Configuration::get('EPSILON_JC_STORE_NAME')
            ),
            'EPSILON_JC_STORE_URL' => Tools::getValue(
                'EPSILON_JC_STORE_URL',
                Configuration::get('EPSILON_JC_STORE_URL')
            ),
            'EPSILON_JC_STORE_EMAIL' => Tools::getValue(
                'EPSILON_JC_STORE_EMAIL',
                Configuration::get('EPSILON_JC_STORE_EMAIL')
            ),
            'EPSILON_CONTRACT_CODE' => Tools::getValue(
                'EPSILON_CONTRACT_CODE',
                Configuration::get('EPSILON_CONTRACT_CODE')
            ),
            'EPSILON_TEL' => Tools::getValue(
                'EPSILON_TEL',
                Configuration::get('EPSILON_TEL')
            ),
            'EPSILON_JC_DEBUG_URLS_1' => Tools::getValue(
                'EPSILON_JC_DEBUG_URLS',
                Configuration::get('EPSILON_JC_DEBUG_URLS')
            ),
            'EPSILON_DEFERRED_PAYMENT_1' => Tools::getValue(
                'EPSILON_DEFERRED_PAYMENT',
                Configuration::get('EPSILON_DEFERRED_PAYMENT')
            ),
            'EPSILON_REVOLVING_PAYMENT_1' => Tools::getValue(
                'EPSILON_REVOLVING_PAYMENT',
                Configuration::get('EPSILON_REVOLVING_PAYMENT')
            ),
            /*
            'EPSILON_2_TIMES_PAY_1' => Tools::getValue(
                'EPSILON_2_TIMES_PAY',
                Configuration::get('EPSILON_2_TIMES_PAY')
            ),
            */
            'EPSILON_3DS_CHECK_CODE_1' => Tools::getValue(
                'EPSILON_3DS_CHECK_CODE',
                Configuration::get('EPSILON_3DS_CHECK_CODE')
            ),
            'EPSILON_API_LANGUAGE' => Tools::getValue(
                'EPSILON_API_LANGUAGE',
                Configuration::get('EPSILON_API_LANGUAGE')
            )
        );

        if (!Tools::isSubmit('btnSubmit')) {
            $rs['EPSILON_JC_STORE_ID'] = Tools::getValue(
                'EPSILON_JC_STORE_ID',
                Configuration::get('EPSILON_JC_STORE_ID')
            );
        } else {
            $rs['EPSILON_JC_STORE_ID'] = Configuration::get('EPSILON_JC_STORE_ID');
        }

        return $rs;
    }

    /*
    Saves the first data sent to provider (JC/Epsilon) onto log table
    Not all parameters are save, just the essential data.
    */
    private function orderSave($data)
    {

        $status = self::TRANSACTION_REQUEST;

        $rs = array();
        $rs['error'] = false;
        $rs['id'] = $this->newID();

        Db::getInstance()->insert(
            'epsilon_client_order',
            array(
                'id' => $rs['id'],
                'ps_id_cart' => $data['jc_ps_id_cart'],
                'date_created' => date('Y-m-d H:i:s'),
                'client_order_amount' => $data['item_price'],
                'client_order_id' => $data['order_number'],
                'client_user_id' => $data['user_id'],
                'client_user_email' => $data['user_mail_add'],
                'client_comment' => $data['memo1'].$data['memo2'],
                'client_pay_currency' => $data['currency_id'],
                'client_pay_method' => $data['jc_pay_method'],
                'provider_contract_code' => $data['contract_code'],
                'pay_store_id' => $data['jc_store_id'],
                'pay_provider_id' => $data['jc_service_provider'],
                'pay_provider_action' => $data['jc_action_url'],
                'jc_debug_urls' => $data['jc_debug_urls'],
                'status' => $status
            )
        );

        if ($db_err = Db::getInstance()->getNumberError()) {
            $rs['error'] = 'DB';
            $rs['error_message'] = $db_err.':'.Db::getInstance()->getMsgError();
        }

        return $rs;

    }

    /*
    Saves the response from provider (JC/Epsilon) onto log table
    */
    private function orderSaveProviderResponse($data, $provider)
    {

        $status = self::TRANSACTION_RESPONSE;

        $rs = array();
        $rs['error'] = false;
        $rs['id'] = $this->newID().'_2';

        Db::getInstance()->insert(
            'epsilon_client_order',
            array(
                'id' => $rs['id'],
                'date_created' => date('Y-m-d H:i:s'),
                'ps_id_cart' => $data['jc_ps_id_cart'],
                'client_order_id' => $data['order_number'],

                'client_order_amount' => $data['item_price'],
                'client_order_id' => $data['order_number'],
                'client_user_id' => $data['user_id'],
                'client_user_email' => $data['user_mail_add'],
                'client_comment' => $data['memo1'].$data['memo2'],
                'client_pay_currency' => $data['currency_id'],
                'client_pay_method' => $data['jc_pay_method'],

                'provider_contract_code' => $data['contract_code'],
                'pay_provider_transaction_id' => $provider['transaction_id'],
                'pay_provider_status' => $provider['status'],
                'pay_provider_error_code' => $provider['error_code'],
                'pay_provider_error_detail' => $provider['error_detail'],
                'pay_provider_result' => $provider['result'],
                'pay_provider_billing_status' => $provider['billing_status'],
                'acs_url' => $provider['acs_url'],
                'acs_pareq' => $provider['acs_pareq'],
                'pay_store_id' => $data['jc_store_id'],
                'pay_provider_id' => $data['jc_service_provider'],
                'pay_provider_action' => $data['jc_action_url'],
                'jc_debug_urls' => $data['jc_debug_urls'],
                'status' => $status,
            )
        );

        if ($db_err = Db::getInstance()->getNumberError()) {
            $rs['error'] = 'DB';
            $rs['error_message'] = $db_err.':'.Db::getInstance()->getMsgError();
        }

        return $rs;

    }
    
    /*
    Verifies if an request already made with given cart id.
    */
    public function checkTransactionRequested($id_cart)
    {

        $sql = 'SELECT 
        T2.id_order AS '.'ps'.'_id_order
        FROM `'._DB_PREFIX_.'orders` AS T2 
        WHERE 
        T2.id_cart = '.($id_cart);

        //echo $sql; exit;
        $rs = Db::getInstance()->getRow($sql);
        if (!empty($rs)) {
            Tools::redirect($this->context->link->getPageLink('history.php', true));
            exit;
        }
    }
    
    /*
    Generates unique id based on microtime
    */
    public function newID()
    {
        /**
        http://stackoverflow.com/questions/307486/short-unique-id-in-php
        */
        $a = explode(' ', microtime());
        return base_convert(str_replace('.', '', $a[0]), 10, 36).base_convert($a[1], 10, 36);
    }

    /*
    Display Epsilon tab on admin product page.
    http://nemops.com/prestashop-products-new-tabs-fields/#.VXaluM-qpBc

    public function hookDisplayAdminProductsExtra($params)
    {
        echo 11111;
        //return $this->display(__FILE__, 'views/admin/MyTab.tpl');
    }
    public function hookActionProductUpdate($params) {
    }
    */

    /*
    Retrieves the options and entire transaction logs to display on order detail page.
    */
    public function hookDisplayAdminOrder($params)
    {
        $order = new Order($params['id_order']);
        //echo $order->current_state; exit;

        /*
        Retrieving the last log.
        */
        $sql = 'SELECT 
        T2.date_created AS date_created
        ,T2.client_pay_method AS client_pay_method
        ,T2.pay_store_id AS pay_store_id
        ,T2.pay_provider_id AS pay_provider_id
        ,T2.client_order_id AS client_order_id
        ,T2.client_order_amount AS client_order_amount
        ,T2.provider_contract_code AS provider_contract_code
        ,T2.pay_provider_billing_status AS pay_provider_billing_status
        ,T2.pay_provider_result AS pay_provider_result
        ,T2.pay_provider_status AS pay_provider_status
        ,T2.pay_provider_action AS pay_provider_action
        ,T2.pay_provider_error_code AS pay_provider_error_code
        ,T2.pay_provider_error_detail AS pay_provider_error_detail
        ,T2.provider_contract_code AS provider_contract_code
        ,T2.jc_debug_urls AS jc_debug_urls
        ,T2.status AS `status`
        ,T2.acs_pareq AS `acs_pareq`
        ,T2.acs_pares AS `acs_pares`
        FROM `'._DB_PREFIX_.'epsilon_client_order` AS T2 
        WHERE 
        T2.ps_id_cart = '.(int)($order->id_cart).'
        ORDER BY T2.date_created DESC
        ';

        //echo $sql; exit;
        //$logs = Db::getInstance()->getRow($sql);
        $logs = Db::getInstance()->executeS($sql);
        //print_r($logs); exit;

        /*
        Check if payment was canceled, regardless of order state.
        */
        $sql = 'SELECT 
        T2.id AS id,
        T2.date_created AS date_created
        FROM `'._DB_PREFIX_.'epsilon_client_order` AS T2 
        WHERE 
        T2.ps_id_cart = '.(int)($order->id_cart).'
        AND
        T2.status = '.self::TRANSACTION_CANCEL_RESPONSE.'
        AND
        T2.pay_provider_result = 1
        ';

        //echo $sql; exit;
        $epsilon_state_canceled = Db::getInstance()->getRow($sql);
        //print_r($epsilon_state_canceled); exit;

        if ($logs) {

            $cards = $this->runningCards();
            //echo $cards[$logs[0]['client_pay_method']]['brand']; exit;

            /*
            echo $this->context->link->getModuleLink($this->name,'AmountChange') . '<br />';
            echo $this->context->link->getAdminLink('EpsilonAmountChange', true) . '<br />';
            exit;
            */
            /**
            $arr = array('テスト','環境');
            $str = Tools::jsonEncode($arr);
            echo 'a: '.$str.PHP_EOL; print_r(Tools::jsonDecode($str, true)); exit;
            */
            //$this->smarty->assign('ajax_url_base', $this->ajax_url_base);
            $this->smarty->assign('ajax_token', Tools::getAdminTokenLite('AdminModules'));
            $this->smarty->assign('id_cart', $order->id_cart);
            $this->smarty->assign('epsilon_log', $logs);
            $this->smarty->assign('epsilon_card_brands', $cards);
            //$base = Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/';
            //$base = str_replace(array('http://', 'https://'), '//', $base);
            //$this->smarty->assign('url_base', $base);
            //$this->smarty->assign('url_base', '//'._PS_BASE_URL_.__PS_BASE_URI__.'modules/'.$this->name.'/');
            //$this->smarty->assign('epsilon_token', Tools::getToken(false));
            $this->smarty->assign('epsilon_state_canceled', (
                !empty($epsilon_state_canceled) ? $epsilon_state_canceled : false
            ));

            $this->smarty->assign('epsilon_transaction_status_billed', self::TRANSACTION_STATUS_BILLED);
            $this->smarty->assign('epsilon_transaction_request', self::TRANSACTION_REQUEST);
            $this->smarty->assign('epsilon_transaction_response', self::TRANSACTION_RESPONSE);
            $this->smarty->assign('epsilon_transaction_amount_change_request', self::TRANSACTION_AMOUNT_CHANGE_REQUEST);
            $this->smarty->assign('epsilon_transaction_cancel_request', self::TRANSACTION_CANCEL_REQUEST);
            $this->smarty->assign('epsilon_transaction_real_sale_request', self::TRANSACTION_REAL_SALE_REQUEST);
            $this->smarty->assign('epsilon_transaction_real_sale_response', self::TRANSACTION_REAL_SALE_RESPONSE);
            $this->smarty->assign('epsilon_transaction_3ds_pares_request', self::TRANSACTION_3DS_PARES_REQUEST);
            $this->smarty->assign('epsilon_transaction_3ds_pares_response', self::TRANSACTION_3DS_PARES_RESPONSE);
            return $this->display(__FILE__, 'admin-order.tpl');
        }

        return null;

    }

    /*
    Send real sale request to Epsilon
    */
    public function ajaxProcessRealSaleRequest($ps_id_cart = null)
    {

        if (empty($ps_id_cart)) {
            $ps_id_cart = Tools::getValue('ps_id_cart');
        }

        $rs = array();
        $rs['error'] = false;

        include_once(dirname(__FILE__).'/lib/JapanCase.php');
        $jc = new JapanCase();

        $sql = 'SELECT 
        *
        FROM `'._DB_PREFIX_.'epsilon_client_order`
        WHERE 
        ps_id_cart = '.$ps_id_cart.'
        AND
        client_order_amount <> \'\'
        AND
        (pay_provider_result = 1 OR pay_provider_result = 5)
        AND 
        (status = '.self::TRANSACTION_RESPONSE.' OR status = '.self::TRANSACTION_AMOUNT_CHANGE_RESPONSE.')
        ORDER BY date_created DESC
        ';

        //echo $sql; exit;
        if ($db = Db::getInstance()->getRow($sql)) {

            //print_r($db); exit;

            /*
            Saving on log table
            */

            Db::getInstance()->insert(
                'epsilon_client_order',
                array(
                    'id' => $this->newID(),
                    'ps_id_cart' => $ps_id_cart,
                    'date_created' => date('Y-m-d H:i:s'),
                    'client_order_amount' => $db['client_order_amount'],
                    'client_order_id' => $db['client_order_id'],
                    'client_user_id' => $db['client_user_id'],
                    'client_user_email' => $db['client_user_email'],
                    'client_comment' => $db['client_comment'],
                    'client_pay_currency' => $db['client_pay_currency'],
                    'client_pay_method' => $db['client_pay_method'],
                    'provider_contract_code' => $db['provider_contract_code'],
                    'pay_store_id' => $db['pay_store_id'],
                    'pay_provider_id' => $db['pay_provider_id'],
                    'pay_provider_action' => self::EPSILON_API_URL_SALES_PAYMENT, //$db['pay_provider_action'],
                    'jc_debug_urls' => $db['jc_debug_urls'],
                    'status' => self::TRANSACTION_REAL_SALE_REQUEST
                )
            );

            if ($db_err = Db::getInstance()->getNumberError()) {
                $rs['error'] = 'DB';
                $rs['error_message'] = $db_err.':'.Db::getInstance()->getMsgError();
            }

        } else {

            $rs['error'] = 'DB';
            $rs['error_message'] = $this->l('No entries found.');

        }

        //$rs['debug'] = $db;
        //return $rs;

        if (!$rs['error']) {

            /*
            Requesting the JapanCase/Epsilon API
            */
            $rs['gateway'] = $jc->RealSale(
                $db['pay_provider_id'],
                $db['provider_contract_code'],
                $db['pay_store_id'],
                $db['client_order_id'],
                $db['jc_debug_urls']
            );

            if ($rs['gateway']->error) {
                $rs['error'] = $rs['gateway']->error;
                $rs['error_message'] = $this->l('Gateway error');
            }

        }

        if (!$rs['error']) {

            /*
            Saving Espilon response on log table
            */
            Db::getInstance()->insert(
                'epsilon_client_order',
                array(
                    'id' => $this->newID(),
                    'ps_id_cart' => $ps_id_cart,
                    'date_created' => date('Y-m-d H:i:s'),
                    'client_order_amount' => $db['client_order_amount'],
                    'client_order_id' => $db['client_order_id'],
                    'client_user_id' => $db['client_user_id'],
                    'client_user_email' => $db['client_user_email'],
                    'client_comment' => $db['client_comment'],
                    'client_pay_currency' => $db['client_pay_currency'],
                    'client_pay_method' => $db['client_pay_method'],
                    'provider_contract_code' => $db['provider_contract_code'],
                    'pay_store_id' => $db['pay_store_id'],
                    'pay_provider_id' => $db['pay_provider_id'],
                    'pay_provider_action' => self::EPSILON_API_URL_SALES_PAYMENT, //$db['pay_provider_action'],
                    'jc_debug_urls' => $db['jc_debug_urls'],
                    'status' => self::TRANSACTION_REAL_SALE_RESPONSE,

                    'pay_provider_status' => $rs['gateway']->provider['status'],
                    'pay_provider_error_code' => $rs['gateway']->provider['error_code'],
                    'pay_provider_error_detail' => $rs['gateway']->provider['error_detail'],
                    'pay_provider_result' => $rs['gateway']->provider['result'],
                    'pay_provider_billing_status' => $rs['gateway']->provider['billing_status']
                )
            );

            if ($db_err = Db::getInstance()->getNumberError()) {
                $rs['error'] = 'DB';
                $rs['error_message'] = $db_err.':'.Db::getInstance()->getMsgError();
            }

            /*
            Payment billed
            Must refresh the page
            */
            /*
            if ($rs['gateway']->provider['result'] == 1)
            {
                $order = new Order(Tools::getValue('id_order'));
                $order->setCurrentState(Configuration::get('EPSILON_OS_ACCEPTED'));
            }
            */

        }

        echo Tools::jsonEncode($rs);
        exit;

    }

    /*
    Send amount change request to Epsilon
    */
    public function ajaxProcessAmountChangeRequest($ps_id_cart = null)
    {

        if (empty($ps_id_cart)) {
            $ps_id_cart = Tools::getValue('ps_id_cart');
        }

        $rs = array();
        $rs['error'] = false;

        /*
        Retrieving the order cart object
        */
        $cart = new Cart($ps_id_cart);
        if ($cart) {
            //echo $cart->getOrderTotal(false, Cart::BOTH); exit;
            //echo $cart->id_customer; exit;
            //print_r($db); exit;

            /*
            [disabled]
            If change add products on existing order, this method do not show the new total.
            */
            //$new_amount = $cart->getOrderTotal(true, Cart::BOTH);

            $sql = 'SELECT 
            total_paid,
            total_paid_tax_incl,
            total_paid_real
            FROM `'._DB_PREFIX_.'orders`
            WHERE 
            id_cart = '.$ps_id_cart.'
            ';

            if (!$orders = Db::getInstance()->getRow($sql)) {
                $rs['error'] = $this->l('DataBase error');
                return $rs;
            }

            $new_amount = $orders['total_paid'];


        } else {
            $rs['error'] = 'id_cart';
            $rs['error_message'] = $this->l('Invalid Cart ID');
            return $rs;
        }


        $sql = 'SELECT 
        *
        FROM `'._DB_PREFIX_.'epsilon_client_order`
        WHERE 
        ps_id_cart = '.$ps_id_cart.'
        AND
        client_order_amount <> \'\'
        AND
        (pay_provider_result = 1 OR pay_provider_result = 5)
        AND
        (status = '.self::TRANSACTION_RESPONSE.' OR status = '.self::TRANSACTION_AMOUNT_CHANGE_RESPONSE.')
        ORDER BY date_created DESC
        ';

        //echo $sql; exit;
        if (!$db = Db::getInstance()->getRow($sql)) {
            $rs['error'] = 'DB';
            $rs['error_message'] = $this->l('No entries found.');
            return $rs;
        }
        //return number_format($new_amount,6,'.','').' '.number_format($db['client_order_amount'],6,'.',''); //exit;
        if (number_format($new_amount, 6, '.', '') == number_format($db['client_order_amount'], 6, '.', '')) {
            $rs['error'] = 'order_amount';
            $rs['error_message'] = $this->l('The new order amount is the same value of current order');
            return $rs;
        }

        /*
        Saving on log table
        */
        $db['pay_provider_action'] = self::EPSILON_API_URL_CHANGE_AMOUNT;
        $db['client_order_amount'] = $new_amount;

        $arr = array(
            'id' => $this->newID(),
            'ps_id_cart' => $ps_id_cart,
            'date_created' => date('Y-m-d H:i:s'),
            'client_order_amount' => $db['client_order_amount'],
            'client_order_id' => $db['client_order_id'],
            'client_user_id' => $db['client_user_id'],
            'client_user_email' => $db['client_user_email'],
            'client_comment' => $db['client_comment'],
            'client_pay_currency' => $db['client_pay_currency'],
            'client_pay_method' => $db['client_pay_method'],
            'provider_contract_code' => $db['provider_contract_code'],
            'pay_store_id' => $db['pay_store_id'],
            'pay_provider_id' => $db['pay_provider_id'],
            'pay_provider_action' => $db['pay_provider_action'],
            'jc_debug_urls' => $db['jc_debug_urls'],
            'status' => self::TRANSACTION_AMOUNT_CHANGE_REQUEST
        );

        Db::getInstance()->insert('epsilon_client_order', $arr);

        if ($db_err = Db::getInstance()->getNumberError()) {
            $rs['error'] = 'DB';
            $rs['error_message'] = $db_err.':'.Db::getInstance()->getMsgError();
            return $rs;
        }

        //$rs['debug'] = $db;
        //return $rs;

        /*
        Requesting the JapanCase/Epsilon API
        */
        include_once(dirname(__FILE__).'/lib/JapanCase.php');
        $jc = new JapanCase();

        $rs['gateway'] = $jc->AmountChange(
            $db['pay_provider_id'],
            $db['provider_contract_code'],
            $db['pay_store_id'],
            $db['client_order_id'],
            $db['jc_debug_urls'],
            $db['client_user_id'],
            self::ITEM_CODE,
            $db['client_order_amount']
        );

        if ($rs['gateway']->error) {
            $rs['error'] = $rs['gateway']->error;
            $rs['error_message'] = $this->l('Gateway error');
            return $rs;
        }

        /*
        Saving Espilon response on log table
        */

        $arr['id'] = $this->newID().'_2';
        $arr['date_created'] = date('Y-m-d H:i:s');
        $arr['status'] = self::TRANSACTION_AMOUNT_CHANGE_RESPONSE;
        $arr['pay_provider_status'] = $rs['gateway']->provider['status'];
        $arr['pay_provider_error_code'] = $rs['gateway']->provider['error_code'];
        $arr['pay_provider_error_detail'] = $rs['gateway']->provider['error_detail'];
        $arr['pay_provider_result'] = $rs['gateway']->provider['result'];
        $arr['pay_provider_billing_status'] = $rs['gateway']->provider['billing_status'];

        Db::getInstance()->insert('epsilon_client_order', $arr);

        if ($db_err = Db::getInstance()->getNumberError()) {
            $rs['error'] = 'DB';
            $rs['error_message'] = $db_err.':'.Db::getInstance()->getMsgError();
        }

        echo Tools::jsonEncode($rs);
        exit;

    }

    /*
    Send cancel request to Epsilon
    */
    public function ajaxProcessCancelRequest($ps_id_cart = null)
    {

        if (empty($ps_id_cart)) {
            $ps_id_cart = Tools::getValue('ps_id_cart');
        }

        $rs = array();
        $rs['error'] = false;

        $sql = 'SELECT 
        *
        FROM `'._DB_PREFIX_.'epsilon_client_order`
        WHERE 
        ps_id_cart = '.$ps_id_cart.'
        AND
        client_order_amount <> \'\'
        AND
        (pay_provider_result = 1 OR pay_provider_result = 5)
        AND
        (status = '.self::TRANSACTION_RESPONSE.' OR status = '.self::TRANSACTION_AMOUNT_CHANGE_RESPONSE.')
        ORDER BY date_created DESC
        ';

        //echo $sql; exit;
        if (!$db = Db::getInstance()->getRow($sql)) {
            $rs['error'] = $this->l('DataBase error');
            return $rs;
        }

        /*
        Saving on log table
        */
        $db['pay_provider_action'] = self::EPSILON_API_URL_CANCEL;

        $arr = array(
            'id' => $this->newID(),
            'ps_id_cart' => $ps_id_cart,
            'date_created' => date('Y-m-d H:i:s'),
            'client_order_amount' => $db['client_order_amount'],
            'client_order_id' => $db['client_order_id'],
            'client_user_id' => $db['client_user_id'],
            'client_user_email' => $db['client_user_email'],
            'client_comment' => $db['client_comment'],
            'client_pay_currency' => $db['client_pay_currency'],
            'client_pay_method' => $db['client_pay_method'],
            'provider_contract_code' => $db['provider_contract_code'],
            'pay_store_id' => $db['pay_store_id'],
            'pay_provider_id' => $db['pay_provider_id'],
            'pay_provider_action' => $db['pay_provider_action'],
            'jc_debug_urls' => $db['jc_debug_urls'],
            'status' => self::TRANSACTION_CANCEL_REQUEST
        );

        /*
        Workaround to avoid database errors when these parameters has empty value
        */
        if (empty($db['client_order_amount'])) {
            unset($arr['client_order_amount']);
        }
        if (empty($db['client_pay_method'])) {
            unset($arr['client_pay_method']);
        }

        Db::getInstance()->insert('epsilon_client_order', $arr);
        //unset($arr);

        if ($db_err = Db::getInstance()->getNumberError()) {
            $rs['error'] = 'DB';
            $rs['error_message'] = $db_err.':'.Db::getInstance()->getMsgError();
            return $rs;
        }

        //$rs['debug'] = $db;
        //return $rs;

        /*
        Requesting the JapanCase/Epsilon API
        */
        include_once(dirname(__FILE__).'/lib/JapanCase.php');
        $jc = new JapanCase();

        $rs['gateway'] = $jc->Cancel(
            $db['pay_provider_id'],
            $db['provider_contract_code'],
            $db['pay_store_id'],
            $db['client_order_id'],
            $db['jc_debug_urls']
        );

        if ($rs['gateway']->error) {
            $rs['error'] = $rs['gateway']->error;
            $rs['error_message'] = $this->l('Gateway error');
            return $rs;
        }

        /*
        Saving Espilon response on log table
        */
        $arr['id'] = $this->newID().'_2';
        $arr['date_created'] = date('Y-m-d H:i:s');
        $arr['status'] = self::TRANSACTION_CANCEL_RESPONSE;
        $arr['pay_provider_status'] = $rs['gateway']->provider['status'];
        $arr['pay_provider_error_code'] = $rs['gateway']->provider['error_code'];
        $arr['pay_provider_error_detail'] = $rs['gateway']->provider['error_detail'];
        $arr['pay_provider_result'] = $rs['gateway']->provider['result'];
        $arr['pay_provider_billing_status'] = $rs['gateway']->provider['billing_status'];

        Db::getInstance()->insert('epsilon_client_order', $arr);

        if ($db_err = Db::getInstance()->getNumberError()) {
            $rs['error'] = 'DB';
            $rs['error_message'] = $db_err.':'.Db::getInstance()->getMsgError();
        }

        /*
        Payment billed
        Must refresh the page
        */
        /*
        if ($rs['gateway']->provider['result'] == 1)
        {
            $order = new Order(Tools::getValue('id_order'));
            $order->setCurrentState(self::PS_OS_CANCELED);
        }
        */

        echo Tools::jsonEncode($rs);
        exit;

    }

    public function ACS3DSecureSave($md, $acs_pares)
    {

        /*
        Extracting id_cart and jc_store_id
        */
        $ar = $this->ACS3DSecureMDUnmount($md);
        $id_cart = $ar[0];
        $order_number = $ar[1];
        $jc_store_id = $ar[2];
        unset($ar);

        $rs = array();
        $rs['error'] = false;

        /*
        Saving on log table
        */
        $rs['data']['p'] = 'Transaction/Send';
        $rs['data']['order_number'] = $order_number;
        $rs['data']['tds_pares'] = $acs_pares;
        $rs['data']['tds_check_code'] = self::TDS_CHECK_CODE_VALIDATE_PARES;
        $rs['data']['contract_code'] = Configuration::get('EPSILON_CONTRACT_CODE');
        $rs['data']['jc_store_id'] = $jc_store_id;
        $rs['data']['jc_service_provider'] = $this->jc_service_provider;
        $rs['data']['jc_action_url'] = self::EPSILON_API_URL_CARD_PAYMENT;
        $rs['data']['jc_debug_urls'] = Configuration::get('EPSILON_JC_DEBUG_URLS');
        $rs['data']['jc_ps_id_cart'] = $id_cart;

        $arr = array(
            'id' => $this->newID(),
            'date_created' => date('Y-m-d H:i:s'),
            'ps_id_cart' => $rs['data']['jc_ps_id_cart'],
            'client_order_id' => $rs['data']['order_number'],
            'provider_contract_code' => $rs['data']['contract_code'],
            'pay_store_id' => $rs['data']['jc_store_id'],
            'pay_provider_id' => $rs['data']['jc_service_provider'],
            'pay_provider_action' => $rs['data']['jc_action_url'],
            'jc_debug_urls' => $rs['data']['jc_debug_urls'],
            'acs_pares' => $rs['data']['tds_pares'],
            'status' => self::TRANSACTION_3DS_PARES_REQUEST
        );

        Db::getInstance()->insert('epsilon_client_order', $arr);

        if ($db_err = Db::getInstance()->getNumberError()) {
            $rs['error'] = 'DB';
            $rs['error_message'] = $db_err.':'.Db::getInstance()->getMsgError();
            return $rs;
        }

        /*
        Requesting the JapanCase/Epsilon API
        */
        include_once(dirname(__FILE__).'/lib/JapanCase.php');
        $jc = new JapanCase();
        $rs['gateway'] = $jc->transactionSend($rs['data']);

        //print_r($rs['gateway']->error); exit;
        //echo time(); exit;

        if (!empty($rs['gateway']->error)) {
            $rs['error'] = $rs['gateway']->error;
            $rs['error_message'] = $this->l('Gateway error');
            return $rs;
        }

        /*
        Saving Espilon response on log table
        */
        $arr['id'] = $this->newID().'_2';
        $arr['date_created'] = date('Y-m-d H:i:s');
        $arr['status'] = self::TRANSACTION_3DS_PARES_RESPONSE;
        $arr['pay_provider_status'] = $rs['gateway']->provider['status'];
        $arr['pay_provider_error_code'] = $rs['gateway']->provider['error_code'];
        $arr['pay_provider_error_detail'] = $rs['gateway']->provider['error_detail'];
        $arr['pay_provider_result'] = $rs['gateway']->provider['result'];

        Db::getInstance()->insert('epsilon_client_order', $arr);

        if ($db_err = Db::getInstance()->getNumberError()) {
            $rs['error'] = 'DB';
            $rs['error_message'] = $db_err.':'.Db::getInstance()->getMsgError();
        }

        return $rs;

    }

    public function ACS3DSecureMDMount($id_cart, $order_number, $jc_store_id)
    {
        return $id_cart.':'.$order_number.':'.$jc_store_id;
    }

    public function ACS3DSecureMDUnmount($md)
    {
        return explode(':', $md);
    }
}
