CREATE TABLE `ps_epsilon_client_order` (
  `id` char(30) NOT NULL,
  `status` smallint(3) NOT NULL DEFAULT '10' COMMENT 'See the constants inside Epsilon class\r\n\r\nconst TRANSACTION_*',
  `date_created` datetime NOT NULL,
  `transaction_code` char(40) NOT NULL DEFAULT '0' COMMENT 'Used as "order_id" to send to Epsilon.\r\nThe "client_order_id" uses the id_cart. If return error from provider, cannot send the same id_cart. \r\nThe transaction_code can be changed easily, to solve this issue.',
  `ps_id_cart` char(15) NOT NULL DEFAULT '0' COMMENT 'PrestaShop Cart ID',
  `client_order_id` char(100) NOT NULL COMMENT 'Generated on client store',
  `client_user_id` char(100) DEFAULT NULL COMMENT 'if null, client_user_email will be used',
  `client_user_email` char(130) DEFAULT NULL,
  `client_order_amount` decimal(16,6) DEFAULT NULL,
  `client_order_amount_decimals` tinyint(1) NOT NULL DEFAULT '0' COMMENT '0: int\r\n2: decimal (2 digits)\r\n3: decimal (3 digits)\r\n4: decimal (4 digits)\r\n5: and so..',
  `client_comment` varchar(2000) DEFAULT NULL COMMENT 'Optional notes from client',
  `client_pay_currency` char(3) DEFAULT NULL COMMENT 'Three chars currencies codes ISO 4217\r\nhttp://en.wikipedia.org/wiki/ISO_4217',
  `client_pay_method` tinyint(2) DEFAULT NULL COMMENT '100~199: credit card (referer to credit cards list)\r\neg:\r\n101: visa\r\n102: mastercard\r\n103: jcb\r\n\r\nIf only "100", means that a new credit card provider was sent.\r\n\r\n200: cash on delivery\r\n201: cash on store\r\n\r\n300: bank wire\r\n\r\n400: convenience store\r\n',
  `provider_contract_code` char(50) DEFAULT NULL COMMENT 'Each store can have multiple providers.\r\n\r\nThe fields "pay_transactions_store_id" and "provider_contract_code" cannot be unique for this table.\r\n\r\nThe provider contract code is the contract code between client and provider.',
  `pay_store_id` char(30) NOT NULL DEFAULT 'jcdemo' COMMENT 'The ID of the store/shop requested the transaction.\r\n\r\nThe store must be registered on local database, in the table "pay_transactions_store".',
  `pay_provider_id` char(100) NOT NULL COMMENT 'Provider id',
  `pay_provider_action` char(100) DEFAULT NULL COMMENT 'eg:\r\ndirect_car_payment\r\namount_change\r\nbank_wire\r\ncash_on_delivery\r\ncancel request\r\n\r\n*the action url depends on each provider\r\n*also called as action_url',
  `pay_provider_transaction_id` char(100) DEFAULT '' COMMENT 'transaction code/id sent from provider if exists',
  `pay_provider_status` char(100) DEFAULT '' COMMENT 'status from provider if exists',
  `pay_provider_error_code` char(100) DEFAULT NULL COMMENT 'error code from provider if exists',
  `pay_provider_error_detail` char(255) DEFAULT NULL COMMENT 'error detail from provider if exists',
  `pay_provider_result` char(100) DEFAULT NULL COMMENT 'provider result code or message',
  `pay_provider_billing_status` char(10) DEFAULT NULL COMMENT '0: billed (real sale)\r\n1: provisory (sale waiting confirmation from store)\r\n    can change amount, cancel or set as real sale',
  `acs_url` char(255) DEFAULT NULL COMMENT 'ACS authentication URL if required',
  `acs_pareq` char(100) DEFAULT NULL COMMENT 'ACS authentication parameter sent from provider if required',
  `acs_pares` char(100) DEFAULT NULL COMMENT 'For 3-D Secure\r\n\r\nAuthentication information code to send to payment gateway provider.\r\n\r\nThe payment gateway provider will check the code to complete the process (with error or not).\r\n\r\n',
  `jc_debug_urls` char(1) NOT NULL DEFAULT '0' COMMENT '0: production\r\n1: debug mode (similar to paypal sandbox, just for tests)',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Debug purposes.\r\nSimulates client store''s db table';